@@ -17,16 +17,17 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <arpa/inet.h>
-#include <linux/ax25.h>
-#include <linux/if_ether.h>
+#include <netax25/ax25.h>
+#include <netinet/if_ether.h>
 #include <netinet/in.h>
 #include <netinet/ip_icmp.h>
 #include <netinet/ip.h>
 #include <net/if.h>
-#include <net/if_route.h>
+#include <net/route.h> 
 #include <sys/ioctl.h>
 #include <sys/socket.h> 
 #include <sys/types.h>
+#include <sys/time.h>
 #include <netdb.h>
 #include <string.h>
 #include <stdio.h>
@@ -66,17 +67,21 @@ void send_rrh(char *iface)
 	rrh->type = TYPE_RRH;
 	rrh->checksum = 0;
 	
-	if( (rrh->addr = get_iface_addr(iface)) == INADDR_NONE)
+	if ((saddr = get_mybull_addr()) == INADDR_NONE)
+	{
+		syslog(LOG_DAEMON | LOG_WARNING, "send_rrh(): Cannot get mybull address.");
+		return;
+	}
+	rrh->addr = saddr;
+	if ((saddr = get_iface_addr(iface)) == INADDR_NONE)
 	{
 		syslog(LOG_DAEMON | LOG_WARNING, "send_rrh(): Cannot get interface %s address.", iface);
 		return;
 	}
 	
-	rrh->tx_pkts = htons(get_tx_pkts(iface));
+	rrh->tx_pkts = htons(get_tx_pkts(iface)+1); /* Count this packet */
 	rrh->flags = 1;		/* wizzer - this needs to be changed */
 	
-	saddr = rrh->addr;
-	
 	/* Work out where to send it */
 	if( (daddr = get_bcast_addr(iface)) == INADDR_NONE)
 	{
@@ -87,7 +92,7 @@ void send_rrh(char *iface)
 	
 	/* Calculate the checksum, we need a pseudo header like TCP/UDP :<
 	 */
-	rrh->checksum = rspf_check((char*)rrh, RSPF_RRH_LEN, saddr, daddr);	
+	rrh->checksum = rspf_check((unsigned char*)rrh, RSPF_RRH_LEN, saddr, daddr);	
 	
 	send_rspf(saddr, daddr, outpack, RSPF_RRH_LEN, iface);
 	
@@ -107,25 +112,40 @@ void send_rrh(char *iface)
 int send_ping(u_long daddr, char *iface)
 {	
 	static u_short pingseq = 1;
-	u_char outpack[sizeof(struct icmphdr)];
-	struct icmphdr *icp = (struct icmphdr*)&outpack;
-	int s;
+	u_char *outpack;
+	struct icmphdr *icp;
+	int s, sz, i;
 	struct sockaddr whereto;
 	struct sockaddr_in *to = (struct sockaddr_in*) &whereto;
 	struct rtentry *rtp;
 	struct rtentry rt;
 	static int icmp_protocol = -1;
+	static int ip_protocol = -1;
+	
 	
-
 	/* We cannot have a sequence number of 0 */
 	if (pingseq == 0)
 		pingseq++;
 			
+	
 	/* Create the socket to send the echo on */
 	bzero((char*)&whereto, sizeof(struct sockaddr));
 	to->sin_family = AF_INET;
 	to->sin_addr.s_addr = daddr;
 	
+	if ( ip_protocol < 0 ) {
+		/*
+		 * The protocol won't change while the daemon's running,
+		 * so there's no reason to read /etc/protocols repeatedly.
+		 */
+		struct protoent *proto;
+		if ((proto = getprotobyname("ip")) == NULL) {
+			syslog(LOG_DAEMON | LOG_CRIT,
+			 "send_ping(): unknown protocol ip (%m)");
+			return 0;
+		}
+		ip_protocol = proto->p_proto;
+	}
 	if ( icmp_protocol < 0 ) {
 		/*
 		 * The protocol won't change while the daemon's running,
@@ -144,14 +164,37 @@ int send_ping(u_long daddr, char *iface)
 		return 0;
 	}
 	
+	i = 1;
+	if (setsockopt(s, ip_protocol, IP_TTL, (char*)&i, sizeof(i)) < 0) {
+		syslog(LOG_DAEMON | LOG_WARNING, "send_ping(): ioctl to set TTL failed (%m)");
+	}
+	/* Work out the size of packet and make a buffer */
+	if (rspf_stats.rspfPingSize < sizeof(struct icmphdr) )
+		sz = sizeof(struct icmphdr);
+	else
+		sz = rspf_stats.rspfPingSize;
+		
+	if ( (outpack = (unsigned char*)malloc(sz)) == NULL)
+	{
+		syslog(LOG_DAEMON | LOG_CRIT, "send_ping(): malloc for packet failed (%m)");
+		return 0;
+	}
+
+	icp = (struct icmphdr*)outpack;	
 	/* Create ICMP header */
 	icp->type = ICMP_ECHO;
 	icp->code = 0;
 	icp->checksum = 0;
 	icp->un.echo.id = getpid();
 	icp->un.echo.sequence = pingseq++;
-	icp->checksum = in_cksum((u_short*)outpack, sizeof(struct icmphdr));
 
+	/* Fill in the blanks */
+	for (i = sizeof(struct icmphdr); i < sz; i++)
+		outpack[i] = (char)(i-sizeof(struct icmphdr));
+
+	/* Checksum covers entire message (TCPIP Illust pp69) */
+	icp->checksum = in_cksum((u_short*)outpack, sz);
+		
 	/*
 	 * We have to set a dummy route temporarily, if none is there
 	 */
@@ -159,7 +202,7 @@ int send_ping(u_long daddr, char *iface)
 	{
 		rt = *rtp;
 		add_route(daddr, 32, 0, 1);
-		sendto(s, outpack, sizeof(struct icmphdr), 0, &whereto, sizeof(struct sockaddr));
+		sendto(s, outpack, sz, 0, &whereto, sizeof(struct sockaddr));
 		/*
 		 * Restore the route entry
 		 */
@@ -171,12 +214,17 @@ int send_ping(u_long daddr, char *iface)
 	else
 	{
 		/* No route, so we can go for it */
-		add_route(daddr, 32, 0 ,1);
-		sendto(s, outpack, sizeof(struct icmphdr), 0, &whereto, sizeof(struct sockaddr));
+		add_route(daddr, 32, 0, 1);
+		sendto(s, outpack, sz, 0, &whereto, sizeof(struct sockaddr));
 		del_route(daddr, 32);
 	}
-	close(s);
-	return icp->un.echo.sequence;
+	if (close(s) < 0 )
+		syslog(LOG_DAEMON | LOG_ERR, "send_ping(): Close failed (%m).");
+
+	/* We have to do this otherwise we return a free'ed space */
+	i = icp->un.echo.sequence;
+	free(outpack);
+	return i;
 }	
 
 /*
@@ -188,13 +236,12 @@ int send_ping(u_long daddr, char *iface)
  *	Nothing
  *
  * Arguments:
- *	u_long		Source address of link
  *	u_long		Destination address of link
  *	u_char		Significant bits of dest addr
  *	u_char		Cost of link
- *	char*		Name of port (ignored - wizzer remove it)
+ *  u_char		Horizon of link
  */
-void send_news(u_long saddr, u_long daddr, u_char sigbits, u_char cost, char *port)
+void send_news(u_long daddr, u_char sigbits)
 {
 	u_char buf[RSPFNODE_LEN + RSPFLINK_LEN + RSPFADJ_LEN];
 	struct rspfnode_hdr *nodehdr = (struct rspfnode_hdr*)buf;
@@ -204,14 +251,15 @@ void send_news(u_long saddr, u_long daddr, u_char sigbits, u_char cost, char *po
 	char ifbuf[256];
 	char *ifname;
 	int ifcount;
+	u_long addr;
+	u_char cost;
 	
-	bcopy((char*)&saddr, nodehdr->addr, 4);
+	cost = mark_ignored_adjacencies(daddr);
 	nodehdr->seq_no = htons(rspf_stats.SequenceNumber);
 	nodehdr->sub_seq_no = ++rspf_stats.SubSequenceNumber;
 	nodehdr->links = 1;
 	
-	/* Wizzer get horizon */
-	linkhdr->horizon = get_horizon(daddr, port);
+	linkhdr->horizon = get_horizon(daddr, NULL);
 	linkhdr->erp = 0;
 	linkhdr->cost = cost;
 	linkhdr->adjacencies = 1;
@@ -219,12 +267,16 @@ void send_news(u_long saddr, u_long daddr, u_char sigbits, u_char cost, char *po
 	adjhdr->sig_bits = sigbits | RSPFADJ_LASTFLAG;
 	bcopy((char*)&daddr, adjhdr->addr, 4);
 
+	addr = get_mybull_addr();
+	bcopy((char*)&addr, nodehdr->addr, 4);
+
 	/* Send to get stuck in an envelope and send */
 	ifname = ifbuf;
 	ifcount = rspf_ifaces(ifbuf, sizeof(ifbuf));
 	while(ifcount-- > 0)
 	{
 		send_rspf_env(buf, sizeof(buf), 1, ifname);
+		
 		ifname += strlen(ifname) + 1;
 	}
 		
@@ -237,7 +289,23 @@ int send_rspf(u_long saddr, u_long daddr, u_char *buf, int buflen,  char *iface)
 	int i;
 /*	FILE *fp;*/
 	static int rspf_protocol = -1;
+	static int ip_protocol = -1;
 
+	bzero((char*)&sin, sizeof(struct sockaddr_in) );
+	
+	if ( ip_protocol < 0 ) {
+		/*
+		 * The protocol won't change while the daemon's running,
+		 * so there's no reason to read /etc/protocols repeatedly.
+		 */
+		struct protoent *proto;
+		if ((proto = getprotobyname("ip")) == NULL) {
+			syslog(LOG_DAEMON | LOG_CRIT,
+			 "send_ping(): unknown protocol ip (%m)");
+			return 0;
+		}
+		ip_protocol = proto->p_proto;
+	}
 	if ( rspf_protocol < 0 ) {
 		/*
 		 * No need to scan /etc/protocols more than once, and
@@ -252,16 +320,20 @@ int send_rspf(u_long saddr, u_long daddr, u_char *buf, int buflen,  char *iface)
 	}
 			
 	if ( (skt = socket(AF_INET, SOCK_RAW, rspf_protocol)) < 0) {
-		perror("RSPFd: send_rrh(): socket");
-		close(skt);
+		syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): socket failed (%m)");
 		return -1 ;
 	}
 	
+	i = 1;
+	if (setsockopt(skt, ip_protocol, IP_TTL, (char*)&i, sizeof(i)) < 0) {
+		syslog(LOG_DAEMON | LOG_WARNING, "send_rspf(): ioctl to set TTL failed (%m)");
+	}
 	/* We have to tell the system we really want to send a b'cast packet */
 	i = 1;
 	if (setsockopt(skt, SOL_SOCKET, SO_BROADCAST, (char*)&i, sizeof(i)) < 0) {
-		perror("RSPFd: send_rrh(): setsockopt");
-		close(skt);
+		syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): setsockopt failed (%m)");
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): close failed (%m)");
 		return -1;
 	}
 	
@@ -269,7 +341,8 @@ int send_rspf(u_long saddr, u_long daddr, u_char *buf, int buflen,  char *iface)
 	if((sin.sin_addr.s_addr = get_bcast_addr(iface) )  == INADDR_NONE)
 	{
 		syslog(LOG_DAEMON | LOG_WARNING, "send_rspf(): Cannot get broadcast address for interface %s.", iface);
-		close(skt);
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): close failed (%m)");
 		return -1;
 	}
 	
@@ -277,11 +350,13 @@ int send_rspf(u_long saddr, u_long daddr, u_char *buf, int buflen,  char *iface)
 	i = sendto(skt, buf, buflen, 0, (struct sockaddr*)&sin, sizeof(struct sockaddr) );
 	if (i < 0) {
 		syslog(LOG_DAEMON | LOG_ERR,"send_rspf(): sendto failed (%m)");
-		close(skt);
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): close failed (%m)");
 		return -1;
 	}
 	rspf_stats.rspfOutMsgs++;
-	close(skt);
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "send_rspf(): close failed (%m)");
 	return 0; 
 } /* send_rspf */
 
@@ -302,8 +377,7 @@ void send_full_bulletin(char *iface)
 	struct router *rtr;
 	qmark rtr_qm;
 	u_char buf[4096];
-	int bufcnt, nodecnt, tmpbuf, tmpnode, size;
-	u_long addr;
+	int bufcnt, nodecnt, size;
 	
 	char ifbuf[256];
 	char *ifname;
@@ -312,7 +386,9 @@ void send_full_bulletin(char *iface)
 	bufcnt = 0;
 	nodecnt = 0;
 
-
+	if (rspf_stats.SequenceNumber == 0)
+		rspf_stats.SequenceNumber++;
+		
 
 	rspf_stats.SubSequenceNumber = 0;
 
@@ -320,7 +396,7 @@ void send_full_bulletin(char *iface)
 	rtr = (struct router*)qmove_first(routerq, &rtr_qm);
 	while (rtr != NULL)
 	{
-		if ( (size = get_rtr_bull(rtr->addr, buf + bufcnt, sizeof(buf) - bufcnt)) > 0)
+		if ( (size = get_rtr_bull(rtr->addr, buf + bufcnt, sizeof(buf) - bufcnt, 0)) > 0)
 		{
 			bufcnt += size;
 			nodecnt++;
@@ -328,46 +404,29 @@ void send_full_bulletin(char *iface)
 		rtr = (struct router*)qmove_next(routerq, &rtr_qm);		
 	}
 
+	/*
+	 * Generate our own bulletin at the end.
+	 */
+	if ((size = get_my_bull(buf + bufcnt, sizeof(buf) - bufcnt)) > 0) 
+	{
+		nodecnt++;
+		bufcnt += size;
+	}
+
 	/*
 	 * Send it on each interface, if it is not specified
 	 * We have to generate our bulletin each time for this
 	 */
 	if (iface != NULL)
 	{
-		if ((addr = get_iface_addr(iface)) == INADDR_NONE)
-		{
-			syslog(LOG_ERR | LOG_DAEMON, "send_full_bulletin(): Cannot find interface address for %s (%m).", iface);
-		} else {
-			if ((size = get_my_bull(addr, buf + bufcnt, sizeof(buf)- bufcnt)) > 0)
-			{
-				nodecnt++;
-				bufcnt += size;
-			}
-			send_rspf_env(buf, bufcnt, nodecnt, iface);
-		}
+		send_rspf_env(buf, bufcnt, nodecnt, iface);
 	} else {
-		tmpnode = nodecnt;
-		tmpbuf = bufcnt;
-		
 		ifcount = rspf_ifaces(ifbuf, 256);
 		ifname = ifbuf;
 		while (ifcount-- > 0) 
 		{
-			if ((addr = get_iface_addr(ifname)) == INADDR_NONE)
-			{
-				syslog(LOG_ERR | LOG_DAEMON, "send_full_bulletin(): Cannot find interface address for %s (%m).", iface);
-			} else {
-				if ((size = get_my_bull(addr, buf, sizeof(buf))) > 0)
-				{
-					nodecnt++;
-					bufcnt += size;
-				}
-				send_rspf_env(buf, bufcnt, nodecnt, ifname);
-			}
-		
+			send_rspf_env(buf, bufcnt, nodecnt, ifname);
 			ifname += strlen(ifname) + 1;
-			nodecnt = tmpnode;
-			bufcnt = tmpbuf;
 		} /* while ifcount */
 	} /* no specified interface */
 
@@ -375,8 +434,8 @@ void send_full_bulletin(char *iface)
 	
 void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 {
-	char outbuf[2048];
-	char *startbuf, *endbuf;
+	unsigned char outbuf[2048];
+	unsigned char *startbuf, *endbuf;
 	struct rspfroute_hdr *rthdr = (struct rspfroute_hdr*)outbuf;
 	u_long saddr, daddr;
 	struct ifreq ifr;
@@ -388,7 +447,7 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 	struct outfrag *frg;
 	qmark ofrg_qm;
 
-	if (debug_mode)
+	if (debug_mode >= 2)
 		printf("send_rspf_env(): Sending packet %d bytes, %d nodes onto port %s.\n", size, nodecnt, iface);
 	/* Check that we have an interface */
 	if (iface == NULL)
@@ -400,7 +459,6 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) 
 	{
  		syslog(LOG_DAEMON | LOG_ERR, "send_rspf_env(): socket failed. (%m)");
- 		(void) close(skt);
  		return;
  	}
 	/* Fill in what we can in routing envelope header */
@@ -448,12 +506,12 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 		endbuf += RSPFNODE_LEN;
 		if ((long)endbuf - (long)buf > size)
 		{
-			syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Overrun buffer at node %d", nodes + 1);
+			syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Overrun buffer at node %d/%d.", nodes + 1, nodecnt);
 			nodes = 0;
 			break;
 		}			
 		links = nodehdr->links;
-		if (debug_mode)
+		if (debug_mode >= 2)
 			printf("send_rspf_env(): Node %d has %d links, header at %u bytes.\n", nodes + 1, links, (u_int)((long)nodehdr - (long)buf));
 		while(links-- > 0)
 		{
@@ -469,13 +527,13 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 			endbuf += RSPFLINK_LEN;
 			if ((long)endbuf - (long)buf > size)
 			{
-				syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Overrun buffer at node %d, link %d.", nodes + 1, links + 1);
+				syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Overrun buffer at node %d/%d, link %d/%d (%s).", nodes + 1, nodecnt, links + 1, nodehdr->links, in_ntoa(*((u_long*)&(nodehdr->addr))) );
 				links = 0;
 				nodes = 0;
 				break;				
 			}
 			adjs = linkhdr->adjacencies;
-			if (debug_mode)
+			if (debug_mode >= 2)
 				printf("send_rspf_env(): Link %d of node %d has %d adjacencies, header at %u bytes.\n", links + 1, nodes + 1, adjs, (u_int)((long)linkhdr - (long)buf));
 			while(adjs-- > 0)
 			{
@@ -490,7 +548,7 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 			 	endbuf += RSPFADJ_LEN;
 			 	if ((long)endbuf - (long)buf > size)
 			 	{
-			 		syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Oveerun buffer at node %d, link %d, adj %d", nodes + 1, links + 1, adjs + 1);
+			 		syslog(LOG_ERR | LOG_DAEMON, "send_rspf_env(): Overrun buffer at node %d/%d (%s), link %d/%d, adj %d/%d", nodes + 1, nodecnt, in_ntoa(*((u_long*)&(nodehdr->addr))), links + 1, nodehdr->links, adjs + 1, linkhdr->adjacencies);
 					nodes = 0;
 					links = 0;
 					adjs = 0;
@@ -522,7 +580,7 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 		return;
 	}
 
-	if (debug_mode)
+	if (debug_mode >= 2)
 		printf("send_rspf_env(): Have %d fragments.\n", frag - 1);
 		
 	frag_tot = frag - 1;
@@ -541,7 +599,7 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 		rthdr->sync = sync < 256 ? sync : 0;
 		if (frg->datalen > sizeof(outbuf) - RSPFROUTE_LEN)
 		{
-			syslog(LOG_DAEMON | LOG_ERR, "send_rspf_env(): Fragment of %d bytes too big, dropping.");
+			syslog(LOG_DAEMON | LOG_ERR, "send_rspf_env(): Fragment of %d bytes too big, dropping.", frg->datalen);
 		} else {			
 			bcopy(frg->data, outbuf + RSPFROUTE_LEN, frg->datalen);
 			rthdr->checksum = 0;
@@ -554,7 +612,8 @@ void send_rspf_env(u_char *buf, int size, int nodecnt, char *iface)
 		frg = (struct outfrag*)qmove_first(outfragq, &ofrg_qm);
 	}
 	rspf_stats.rspfOutRouteEnvs++;
-	(void) close(skt);
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "send_rspf_env(): close failed (%m)");
 	
 	/* Remove all fragments that are left */
 	while (qmove_first(outfragq, &ofrg_qm) != NULL)
@@ -583,9 +642,37 @@ void add_outfrag(int frag, int sync, u_char *startbuf, u_char *endbuf)
 	frg->data = (u_char*)frg + sizeof(struct outfrag);
 	frg->datalen = size;
 	
-	if (debug_mode)
+	if (debug_mode >= 2)
 		printf("add_outfrag(): Block copying %d bytes into buffer for frag %d.\n", size, frag);
 	bcopy(startbuf, frg->data, size);
 
 	add_qnode(outfragq, (void*)frg, NULL);
 }
+
+void send_poll(u_long addr, char *iface)
+{
+	u_char buf[RSPFNODE_LEN + RSPFLINK_LEN + RSPFADJ_LEN];
+	struct rspfnode_hdr *nodehdr = (struct rspfnode_hdr*)buf;
+	struct rspflink_hdr *linkhdr = (struct rspflink_hdr*)(buf + RSPFNODE_LEN);
+	struct rspfadj_hdr *adjhdr = (struct rspfadj_hdr*)(buf + RSPFNODE_LEN + RSPFLINK_LEN);
+	
+	if (debug_mode)
+		printf("send_poll() to %s on %s.\n", in_ntoa(addr), iface);
+		
+	bcopy((char*)&addr, nodehdr->addr, 4);
+	nodehdr->seq_no = 0;
+	nodehdr->sub_seq_no = 0;
+	nodehdr->links = 0;
+	
+	linkhdr->horizon = 1;
+	linkhdr->erp = 0;
+	linkhdr->cost = 32;
+	linkhdr->adjacencies = 0;
+	
+	adjhdr->sig_bits = 32 | RSPFADJ_LASTFLAG;
+	bzero(adjhdr->addr,4);
+
+	/* Send to get stuck in an envelope and send */
+	send_rspf_env(buf, sizeof(buf), 1, iface);
+		
+}
