@@ -17,6 +17,7 @@
  * 30/06/96 cs  v0.03, nodegroups work, router addres is interface address
  *		use debug flag, use dynamic flag to determine whic routes to
  *		overwrite
+ *  see CHANGES file
  */
 
 #define CONFIG_FILE "/etc/ax25/rspfd.conf"
@@ -26,9 +27,14 @@ unsigned short rspf_check(unsigned char *rh, int len,
 	 unsigned long saddr, unsigned long daddr);
 u_short in_cksum(u_short *addr, int len); 
 u_char get_horizon(u_long addr, char *iface);
-int get_rtr_bull(u_long addr, u_char *buf, int len);
-int get_my_bull(u_long addr, u_char *buf, int len);
+int get_rtr_bull(u_long addr, u_char *buf, int len, u_char sub_seq_no);
+int get_my_bull(u_char *buf, int len);
+u_long get_mybull_addr(void);
+struct rspf_adj *getadjbyip(u_long addr);
 char* in_ntoa(u_long addr);
+u_char mark_ignored_adjacencies(u_long addr);
+void del_link_source(u_long saddr, u_short env_no);
+void do_spf(void);
 
 #define RSPF_VERSION 22
 
@@ -62,7 +68,8 @@ typedef enum {
 	Ok,
 	Tentative,
 	Suspect,
-	Bad
+	Bad,
+	Marginal
 }  AdjStatus;
 	
 
@@ -84,7 +91,8 @@ struct rspfrrh {
 
 
 struct router {
-	u_long	addr;
+	u_long	addr;		/* Address of the interface */
+	u_long  rtr_id;		/* Router ID */
 	u_int	seq_no;
 	u_int	sub_seq_no;
 	time_t	bulltime;
@@ -99,14 +107,19 @@ struct rspf_route {
 
 struct rspf_adj{
 	u_long	addr;			/* Ip address */
+	u_long  rtr_id;			/* Router ID, if it exists */
 	ax25_address	dladdr;		/* AX.25 address */
 	char 	port[IFNAMSIZ];		/* Port that adj was heard on */
 	u_int	rx_pkts;		/* pkt sequence # from /proc */
 	u_int	tx_pkts;		/* # pkts sent from RRH if there*/
 	time_t	rrhtime;		/* Time of last RRH, 0 if none */
+	time_t	lastheard;		/* Last time wqe heard RSPF data */
+	time_t	badtime;		/* Time degraded to Bad, 0 if not */
 	u_char	rx_ratio;		/* Rx / Tx ratio */
 	int cost;
+	int ignore;			/* Ignore if not lowest cost to addr */
 	u_char horizon;
+	u_short	irtt;
 	AdjStatus status;
 };
 
@@ -117,13 +130,14 @@ struct link {
 	u_char	sigbits;
 	u_char	horizon;
 	u_char	cost;
+	u_char	sub_seq_no;
 };
 
 struct path {
-	u_long daddr;
-	u_long adjaddr;
-	u_char sigbits;
-	u_long paddr;
+	u_long daddr;		/* Destination */
+	u_long adjaddr;		/* Adjacenct router to get to destination */
+	u_char sigbits;		/* Significant bits for destination */
+	u_long paddr;		/* Parent IP address */
 	u_char cost;
 	
 };
@@ -135,6 +149,7 @@ struct pings {
 	int 	id;
 	char	count;
 	int	echo_id;
+	struct timeval pinged;
 };
 		
 struct rspfroute_hdr {
@@ -227,6 +242,7 @@ struct  rspf_mib {
 	unsigned long rspfSusPings;		/* # of pings before adjacaency lost */
 	unsigned long rspfSusTimeout;		/* secs before adj considered sus */
 	unsigned long rspfPingTimer;		/* secs between pings for sus timer */
+	unsigned long rspfPingSize;			/* size of ping packet in bytes */
 	unsigned long rspfBullTimer;		/* secs between routing bulletins */
 	unsigned long rspfBullTimeout;		/* secs before we ignore prev seq # */
 	unsigned long rspfCurrAdjacencies;	/* # of adjacencies we know about */
@@ -240,7 +256,9 @@ struct  rspf_mib {
 	unsigned char rspfGroupHorizon;		/* Horizon of node groups */
 	unsigned char rspfLocalHorizon;		/* Horizon of non-routers in node-groups */
 	unsigned char rspfPortableHorizon;	/* Horizon of non-routers not in node-groups */
+	
+	unsigned char rspfBadThreshold;		/* Minimum percentage of Rx packets before link is bad */
+	float rspfBadAdjustment;	/* What we adjust quality by for bad link */
 };	
 
 #endif /* rspfd.h */
-
